#include "unity/src/unity.h"
#include "rastertoultra_local.h"
#include "../src/gndefs.h"
#include "magigen.h"
#include <stdlib.h>
#include <string.h>

void setUp(void) {}

void tearDown(void) {}

void free_allocated_mem_in_devdata(DEVDATA* dev){

   free(dev->pso32bitSurface);
   dev->pso32bitSurface = NULL;

   for (int i = 0; i < 4; i++){
      free(dev->lpSplMemFront[i].lpBuffer);
      dev->lpSplMemFront[i].lpBuffer = NULL;
   }

   free(dev->lpCMYIn[0]);
   dev->lpCMYIn[0] = NULL;

   free(dev->lpCMYOut[0]);
   dev->lpCMYOut[0] = NULL;

   free(dev->lpKOut);
   dev->lpKOut = NULL;

   free(dev->lpPageBuffer[0]);
   dev->lpPageBuffer[0] = NULL;
}

/*****************************************************************************/
void test_rp2_dpi(void) {

   DEVDATA dev_data = {0};
   struct settings_ settings = {0};
   cups_page_header2_t cups_header = {0};

   settings.OEM = OEM_PRO360;
   settings.xdpi = 300;
   settings.Printer = PRO600;

   pageSetup(&dev_data, settings, cups_header);

   TEST_ASSERT_EQUAL(642, dev_data.xImage);
   TEST_ASSERT_EQUAL(1013, dev_data.yImage);
   TEST_ASSERT_EQUAL(1013, dev_data.width_pts);
   free_allocated_mem_in_devdata(&dev_data);


   /* Ensure the "width" and "height" of the image, as used in the riopro2 file
    * format, doesn't increase as we increase the density of the print.
    */

   settings.xdpi = 600;

   pageSetup(&dev_data, settings, cups_header);

   TEST_ASSERT_EQUAL(642, dev_data.xImage);
   TEST_ASSERT_EQUAL(2026, dev_data.yImage);
   TEST_ASSERT_EQUAL(1013, dev_data.width_pts);
   free_allocated_mem_in_devdata(&dev_data);


   settings.Printer = MODEL_MAGICARD_PRICECARDPRO_FLEX_PLUS;
   settings.pageType = 3;
   settings.XXL_ImageType = UICBVAL_Extended_Monochrome;
   settings.xdpi = 300;

   pageSetup(&dev_data, settings, cups_header);

   TEST_ASSERT_EQUAL(642, dev_data.xImage);
   TEST_ASSERT_EQUAL(1654, dev_data.yImage);
   TEST_ASSERT_EQUAL(1654, dev_data.width_pts);

   free_allocated_mem_in_devdata(&dev_data);

   /* extended 600dpi */
   settings.xdpi = 600;
   pageSetup(&dev_data, settings, cups_header);

   TEST_ASSERT_EQUAL(642, dev_data.xImage);
   TEST_ASSERT_EQUAL(3308, dev_data.yImage);
   TEST_ASSERT_EQUAL(1654, dev_data.width_pts);

   free_allocated_mem_in_devdata(&dev_data);


   settings.pageType = 2;
   settings.XXL_ImageType = UICBVAL_DoubleImage;
   settings.xdpi = 300;

   pageSetup(&dev_data, settings, cups_header);

   TEST_ASSERT_EQUAL(642, dev_data.xImage);
   TEST_ASSERT_EQUAL(1287, dev_data.yImage);
   TEST_ASSERT_EQUAL(1287, dev_data.width_pts);

   free_allocated_mem_in_devdata(&dev_data);

}

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
int main(void) {
    UNITY_BEGIN();

    RUN_TEST(test_rp2_dpi);

    return UNITY_END();
}
