#include "../src/rp2mag.h"
#include "../src/rp2mag_local.h"
#include "../src/utils.h"
#include "unity/src/unity.h"
#include <stdlib.h>
#include <string.h>

void setUp(void) {}

void tearDown(void) {}

/*****************************************************************************/
void test_rp2mag_set_defaults(void) {
    struct Rp2Mag mag;
    struct Rp2Mag mag_null_initialiased = {0};

    rp2mag_set_defaults(&mag);
    rp2mag_set_defaults(&mag_null_initialiased);

    TEST_ASSERT(mag.settings.coercivity == HICO);
    TEST_ASSERT(mag_null_initialiased.settings.coercivity == HICO);

    TEST_ASSERT(mag.settings.standard == ISO);
    TEST_ASSERT(mag_null_initialiased.settings.standard == ISO);

    for (size_t i = 0; i < RP2_MAG_NUM_TRACKS; i++) {
        if (i == 0) {
            TEST_ASSERT(mag.tracks[i].settings.bpc == BPC7);
            TEST_ASSERT(mag_null_initialiased.tracks[i].settings.bpc == BPC7);
        } else {
            TEST_ASSERT(mag.tracks[i].settings.bpc == BPC5);
            TEST_ASSERT(mag_null_initialiased.tracks[i].settings.bpc == BPC5);
        }
        if (i != 1) {
            TEST_ASSERT(mag.tracks[i].settings.bpi == BPI210);
            TEST_ASSERT(mag_null_initialiased.tracks[i].settings.bpi == BPI210);
        } else {
            TEST_ASSERT(mag.tracks[i].settings.bpi == BPI75);
            TEST_ASSERT(mag_null_initialiased.tracks[i].settings.bpi == BPI75);
        }
    }
}

/*****************************************************************************/
void test_rp2mag_extract_track_data_unordered(void) {
    struct Rp2Mag mag = {0};
    char          track_string[] =
            "TRACK3:~3,;7171717171?=true TRACK2:~2,;06121983?=true "
            "TRACK1:~1,%JAMIEROBERTSON?=true";
    rp2mag_extract_raw_track_data_from_string(mag.tracks, track_string);
    rp2mag_raw_track_to_settings(&mag);

    TEST_ASSERT(mag.tracks[0].track_number == 1);
    TEST_ASSERT(mag.tracks[1].track_number == 2);
    TEST_ASSERT(mag.tracks[2].track_number == 3);
}

/*****************************************************************************/
void test_rp2mag_extract_track_data_partial_unordered(void) {
    struct Rp2Mag mag            = {0};
    char          track_string[] = "TRACK3:~3,MPC7;7171717171?=true "
                          "TRACK1:~1,BPI75,%JAMIEROBERTSON?=true";
    rp2mag_extract_raw_track_data_from_string(mag.tracks, track_string);
    rp2mag_raw_track_to_settings(&mag);

    TEST_ASSERT(mag.tracks[0].track_number == 1);
    TEST_ASSERT(mag.tracks[0].settings.bpi == BPI75);
    TEST_ASSERT(mag.tracks[1].track_number == 3);
    TEST_ASSERT(mag.tracks[1].settings.bpc == BPC7);
    TEST_ASSERT(mag.tracks[2].track_number == 0);
}

/*****************************************************************************/
void test_rp2mag_extract_track_data_from_empty_strings(void) {
    char *        empty_strings[] = {"", " ", "           ", "\0"};
    struct Rp2Mag mag             = {0};
    size_t        i               = 0;

    for (; i < sizeof(empty_strings) / sizeof(empty_strings[0]); i++) {
        rp2mag_extract_raw_track_data_from_string(mag.tracks, empty_strings[i]);
        TEST_ASSERT('\0' == mag.tracks[0].data.raw[0]);
        TEST_ASSERT_EQUAL_UINT8(0, mag.tracks[0].track_number);
        TEST_ASSERT('\0' == mag.tracks[1].data.raw[0]);
        TEST_ASSERT_EQUAL_UINT8(0, mag.tracks[1].track_number);
        TEST_ASSERT('\0' == mag.tracks[2].data.raw[0]);
        TEST_ASSERT_EQUAL_UINT8(0, mag.tracks[2].track_number);
    }
}

/*****************************************************************************/
void test_rp2mag_extract_all_track_data_from_string(void) {
    struct Rp2Mag mag = {0};
    char          three_tracks_no_options[] =
            "TRACK1:~1,%TESTDATA?=true TRACK2:~2,;06121983?=true "
            "TRACK3:~3,;7171717171?=true";

    rp2mag_extract_raw_track_data_from_string(mag.tracks,
                                              three_tracks_no_options);
    TEST_ASSERT_EQUAL_STRING("~1,%TESTDATA?", mag.tracks[0].data.raw);
    TEST_ASSERT_EQUAL_STRING("~2,;06121983?", mag.tracks[1].data.raw);
    TEST_ASSERT_EQUAL_STRING("~3,;7171717171?", mag.tracks[2].data.raw);
}

/*****************************************************************************/
void test_rp2mag_extract_single_track_data_from_string(void) {
    struct Rp2Mag mag                       = {0};
    char          single_track_no_options[] = "TRACK1:~1,%TESTDATA?=true";

    rp2mag_extract_raw_track_data_from_string(mag.tracks,
                                              single_track_no_options);
    TEST_ASSERT_EQUAL_STRING("~1,%TESTDATA?", mag.tracks[0].data.raw);
}

/*****************************************************************************/
void test_rp2mag_extract_single_track_data_from_mixed_option_string(void) {
    struct Rp2Mag mag = {0};
    char          single_track_no_options[] =
            "CFOverCoat=0PrintNoOvercoat finishings=3 number-up=1 "
            "TRACK3:~3,;7171717171? "
            "job-uuid=urn:uuid:544f319b-ea91-30bc-45cc-ee889b987932 "
            "job-originating-host-name=localhost date-time-at-creation= "
            "date-time-at-processing= time-at-creation=1635935483 "
            "time-at-processing=1635935483 document-name-supplied=white.png"
            "TRACK3:~3,;7171717171?=true UTDuplex=1Both";

    rp2mag_extract_raw_track_data_from_string(mag.tracks,
                                              single_track_no_options);
    TEST_ASSERT_EQUAL_STRING("~3,;7171717171?", mag.tracks[0].data.raw);

    struct Rp2Mag mag2 = {0};
    char          single_track_no_options2[] =
            "UTDuplex=1Both TRACK2:~2,;7171717171?=true";

    rp2mag_extract_raw_track_data_from_string(mag2.tracks,
                                              single_track_no_options2);
    TEST_ASSERT_EQUAL_STRING("~2,;7171717171?", mag2.tracks[0].data.raw);
}

/*****************************************************************************/
void test_rp2mag_extract_multiple_track_data_from_mixed_option_string(void) {
    struct Rp2Mag mag = {0};
    char          single_track_no_options[] =
            "CFOverCoat=0PrintNoOvercoat TRACK1:~1,%TESTDATA?=true "
            "ColorModel=Grey "
            "TRACK2:~2,;7171717171?=true Resolution=600dpi "
            "TRACK3:~3,;7171717171?=true UTDuplex=1Both";

    rp2mag_extract_raw_track_data_from_string(mag.tracks,
                                              single_track_no_options);
    TEST_ASSERT_EQUAL_STRING("~1,%TESTDATA?", mag.tracks[0].data.raw);
    TEST_ASSERT_EQUAL_STRING("~2,;7171717171?", mag.tracks[1].data.raw);
    TEST_ASSERT_EQUAL_STRING("~3,;7171717171?", mag.tracks[2].data.raw);
}

/*****************************************************************************/
void test_rp2mag_extract_no_track_data_from_non_track_option_string(void) {
    struct Rp2Mag mag = {0};
    char          option_string_with_no_tracks[] =
            "UTDuplex=1Both Resolution=600dpi ColorModel=Grey "
            "CFColourFormat=2MonoK CFOverCoat=0PrintNoOvercoat "
            "CFHoloKote=0PrintNoHoloKote";

    rp2mag_extract_raw_track_data_from_string(mag.tracks,
                                              option_string_with_no_tracks);
    TEST_ASSERT('\0' == mag.tracks[0].data.raw[0]);
}

/*****************************************************************************/
void test_rp2mag_raw_track_to_settings(void) {
    struct Rp2Mag mag = {0};
    mc_strncpy(mag.tracks[0].data.raw,
               "~1,BPI210,MPC5,COEH,%TESTDATA?",
               RP2_MAG_MAX_RAW_LEN);

    rp2mag_raw_track_to_settings(&mag);

    TEST_ASSERT_MESSAGE(BPI210 == mag.tracks[0].settings.bpi,
                        "BPI not extracted from string.");
    TEST_ASSERT_MESSAGE(BPC5 == mag.tracks[0].settings.bpc,
                        "Bits per char not extracted from string");
    TEST_ASSERT_MESSAGE(HICO == mag.settings.coercivity,
                        "Coercivity not extracted from string");
}

/*****************************************************************************/
void test_rp2mag_mulitple_raw_track_to_settings(void) {
    struct Rp2Mag mag = {0};
    mc_strncpy(mag.tracks[0].data.raw,
               "~1,BPI210,MPC5,COEH,%TESTDATA?",
               RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(
            mag.tracks[1].data.raw, "~2,BPI75,;06121983?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[2].data.raw,
               "~3,BPI210,MPC8,;7171717171?",
               RP2_MAG_MAX_RAW_LEN);
    rp2mag_raw_track_to_settings(&mag);
    TEST_ASSERT(HICO == mag.settings.coercivity);

    TEST_ASSERT(1 == mag.tracks[0].track_number);
    TEST_ASSERT(BPI210 == mag.tracks[0].settings.bpi);
    TEST_ASSERT(BPC5 == mag.tracks[0].settings.bpc);
    TEST_ASSERT(2 == mag.tracks[1].track_number);
    TEST_ASSERT(BPI75 == mag.tracks[1].settings.bpi);
    TEST_ASSERT(3 == mag.tracks[2].track_number);
    TEST_ASSERT(BPI210 == mag.tracks[2].settings.bpi);
    TEST_ASSERT(BPC8 == mag.tracks[2].settings.bpc);
}

/*****************************************************************************/
void test_rp2mag_raw_track_to_settings_verify(void) {
    char          xml1[RP2_MAG_MAX_XML_LEN] = {0};
    char          xml2[RP2_MAG_MAX_XML_LEN] = {0};
    struct Rp2Mag mag1                      = {0};
    struct Rp2Mag mag2                      = {0};

    /* Verify */
    mc_strncpy(mag1.tracks[0].data.raw,
               "~1,MGVON,MPC5,COEH,%TESTDATA?",
               RP2_MAG_MAX_RAW_LEN);

    rp2mag_raw_track_to_settings(&mag1);
    TEST_ASSERT(true == mag1.settings.verify);

    rp2mag_encode_tracks(&mag1);
    rp2mag_to_xml(&mag1, xml1);

    TEST_ASSERT_NOT_NULL(strstr(xml1, "<verify-attempts>3</verify-attempts>"));

    /* No Verify */
    mc_strncpy(mag2.tracks[0].data.raw,
               "~1,MGVOFF,MPC5,COEH,%TESTDATA?",
               RP2_MAG_MAX_RAW_LEN);

    rp2mag_raw_track_to_settings(&mag2);
    TEST_ASSERT(false == mag2.settings.verify);

    rp2mag_encode_tracks(&mag2);
    rp2mag_to_xml(&mag2, xml2);

    TEST_ASSERT_NOT_NULL(strstr(xml2, "<verify-attempts>0</verify-attempts>"));
}

/*****************************************************************************/
void test_rp2mag_encode_iso_track(void) {
    struct Rp2Mag mag = {0};
    rp2mag_set_defaults(&mag);
    mc_strncpy(
            mag.tracks[0].data.raw, "~1,COEH,%TESTDATA?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[1].data.raw, "~2,;06121983?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[2].data.raw, "~3,;7171717171?", RP2_MAG_MAX_RAW_LEN);

    /* populate the track numbers from the raw data strings */
    rp2mag_raw_track_to_settings(&mag);
    rp2mag_encode_tracks(&mag);

    TEST_ASSERT_EQUAL_size_t(77, mag.tracks[0].data.encoded_len_bits);
    TEST_ASSERT_EQUAL_size_t(55, mag.tracks[1].data.encoded_len_bits);
    TEST_ASSERT_EQUAL_size_t(65, mag.tracks[2].data.encoded_len_bits);

    TEST_ASSERT_EQUAL_HEX8(0xa2, mag.tracks[0].data.encoded[0]);
    TEST_ASSERT_EQUAL_HEX8(0x5a, mag.tracks[0].data.encoded[1]);
    TEST_ASSERT_EQUAL_HEX8(0x96, mag.tracks[0].data.encoded[2]);
    TEST_ASSERT_EQUAL_HEX8(0x72, mag.tracks[0].data.encoded[3]);
    TEST_ASSERT_EQUAL_HEX8(0xc4, mag.tracks[0].data.encoded[4]);
    TEST_ASSERT_EQUAL_HEX8(0xe1, mag.tracks[0].data.encoded[5]);
    TEST_ASSERT_EQUAL_HEX8(0x96, mag.tracks[0].data.encoded[6]);
    TEST_ASSERT_EQUAL_HEX8(0x87, mag.tracks[0].data.encoded[7]);
    TEST_ASSERT_EQUAL_HEX8(0xf0, mag.tracks[0].data.encoded[8]);
    TEST_ASSERT_EQUAL_HEX8(0xe0, mag.tracks[0].data.encoded[9]);
    TEST_ASSERT_EQUAL_HEX8(0x00, mag.tracks[0].data.encoded[10]);

    TEST_ASSERT_EQUAL_HEX8(0xd0, mag.tracks[1].data.encoded[0]);
    TEST_ASSERT_EQUAL_HEX8(0x5b, mag.tracks[1].data.encoded[1]);
    TEST_ASSERT_EQUAL_HEX8(0x04, mag.tracks[1].data.encoded[2]);
    TEST_ASSERT_EQUAL_HEX8(0x42, mag.tracks[1].data.encoded[3]);
    TEST_ASSERT_EQUAL_HEX8(0x62, mag.tracks[1].data.encoded[4]);
    TEST_ASSERT_EQUAL_HEX8(0xcf, mag.tracks[1].data.encoded[5]);
    TEST_ASSERT_EQUAL_HEX8(0xd0, mag.tracks[1].data.encoded[6]);
    TEST_ASSERT_EQUAL_HEX8(0x00, mag.tracks[1].data.encoded[7]);

    TEST_ASSERT_EQUAL_HEX8(0xd7, mag.tracks[2].data.encoded[0]);
    TEST_ASSERT_EQUAL_HEX8(0x21, mag.tracks[2].data.encoded[1]);
    TEST_ASSERT_EQUAL_HEX8(0xc8, mag.tracks[2].data.encoded[2]);
    TEST_ASSERT_EQUAL_HEX8(0x72, mag.tracks[2].data.encoded[3]);
    TEST_ASSERT_EQUAL_HEX8(0x1c, mag.tracks[2].data.encoded[4]);
    TEST_ASSERT_EQUAL_HEX8(0x87, mag.tracks[2].data.encoded[5]);
    TEST_ASSERT_EQUAL_HEX8(0x21, mag.tracks[2].data.encoded[6]);
    TEST_ASSERT_EQUAL_HEX8(0xf4, mag.tracks[2].data.encoded[7]);
    TEST_ASSERT_EQUAL_HEX8(0x00, mag.tracks[2].data.encoded[8]);
}

/*****************************************************************************/
void test_rp2mag_to_xml(void) {
    struct Rp2Mag mag                      = {0};
    char          xml[RP2_MAG_MAX_XML_LEN] = {0};
    rp2mag_set_defaults(&mag);

    /* make sure nothing is output if there is no mag data */
    rp2mag_to_xml(&mag, xml);
    TEST_ASSERT_EQUAL_STRING("", xml);

    mc_strncpy(mag.tracks[0].data.raw, "~1,%TESTDATA?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[1].data.raw, "~2,;06121983?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[2].data.raw, "~3,;7171717171?", RP2_MAG_MAX_RAW_LEN);

    /* populate the track numbers from the raw data strings */
    rp2mag_raw_track_to_settings(&mag);
    rp2mag_encode_tracks(&mag);

    rp2mag_to_xml(&mag, xml);

    TEST_ASSERT_NOT_NULL(strstr(xml, "<mag-options>"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "</mag-options>"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "<track-options track=\"1\">"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "<track-options track=\"2\">"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "<track-options track=\"3\">"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "</track-options>"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "<num-bits>"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "</num-bits>"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "<bpi>"));
    TEST_ASSERT_NOT_NULL(strstr(xml, "</bpi>"));
}

/*****************************************************************************/
void test_rp2mag_total_encoded_as_bytes(void) {
    struct Rp2Mag mag = {0};
    rp2mag_set_defaults(&mag);
    mc_strncpy(mag.tracks[0].data.raw, "~1,%TESTDATA?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[1].data.raw, "~2,;06121983?", RP2_MAG_MAX_RAW_LEN);
    mc_strncpy(mag.tracks[2].data.raw, "~3,;7171717171?", RP2_MAG_MAX_RAW_LEN);
    rp2mag_raw_track_to_settings(&mag);
    rp2mag_encode_tracks(&mag);

    for (size_t i = 0; i < RP2_MAG_NUM_TRACKS; i++) {
        TEST_ASSERT(mag.tracks[i].data.encoded_len_round_bytes * 8 >
                    mag.tracks[i].data.encoded_len_bits);
    }

    TEST_ASSERT_EQUAL(26, rp2mag_total_encoded_as_bytes(&mag));
}

/*****************************************************************************/
void test_rp2mag_standard(void) {
    struct Rp2Mag mag_iso                      = {0};
    struct Rp2Mag mag_jis                      = {0};
    char          xml_iso[RP2_MAG_MAX_XML_LEN] = {0};
    char          xml_jis[RP2_MAG_MAX_XML_LEN] = {0};

    rp2mag_set_defaults(&mag_iso);
    mc_strncpy(
            mag_iso.tracks[0].data.raw, "~1,%TESTDATA?", RP2_MAG_MAX_RAW_LEN);
    rp2mag_raw_track_to_settings(&mag_iso);
    rp2mag_encode_tracks(&mag_iso);
    rp2mag_to_xml(&mag_iso, xml_iso);

    /* default to ISO */
    TEST_ASSERT_EQUAL(ISO, mag_iso.settings.standard);
    TEST_ASSERT_NOT_NULL(strstr(xml_iso, "<direction>forwards</direction>"));

    /* select JIS */
    rp2mag_set_defaults(&mag_jis);
    mc_strncpy(mag_jis.tracks[0].data.raw,
               "~1,JIS,%TESTDATA?",
               RP2_MAG_MAX_RAW_LEN);
    rp2mag_raw_track_to_settings(&mag_jis);
    rp2mag_encode_tracks(&mag_jis);
    rp2mag_to_xml(&mag_jis, xml_jis);

    TEST_ASSERT_EQUAL(JIS, mag_jis.settings.standard);
    TEST_ASSERT_NOT_NULL(strstr(xml_jis, "<direction>backwards</direction>"));

    TEST_ASSERT(0 < rp2mag_total_encoded_as_bytes(&mag_jis));
    TEST_ASSERT(mag_jis.tracks[0].data.encoded_len_bits <=
                rp2mag_total_encoded_as_bytes(&mag_jis) * 8);
}

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
int main(void) {
    UNITY_BEGIN();
    RUN_TEST(test_rp2mag_set_defaults);

    RUN_TEST(test_rp2mag_extract_track_data_from_empty_strings);
    RUN_TEST(test_rp2mag_extract_single_track_data_from_string);
    RUN_TEST(test_rp2mag_extract_all_track_data_from_string);
    RUN_TEST(test_rp2mag_extract_single_track_data_from_mixed_option_string);
    RUN_TEST(test_rp2mag_extract_multiple_track_data_from_mixed_option_string);
    RUN_TEST(test_rp2mag_extract_no_track_data_from_non_track_option_string);

    RUN_TEST(test_rp2mag_raw_track_to_settings);
    RUN_TEST(test_rp2mag_mulitple_raw_track_to_settings);

    RUN_TEST(test_rp2mag_extract_track_data_unordered);
    RUN_TEST(test_rp2mag_extract_track_data_partial_unordered);
    RUN_TEST(test_rp2mag_encode_iso_track);

    RUN_TEST(test_rp2mag_to_xml);
    RUN_TEST(test_rp2mag_total_encoded_as_bytes);

    RUN_TEST(test_rp2mag_raw_track_to_settings_verify);

    RUN_TEST(test_rp2mag_standard);
    return UNITY_END();
}
